module CanadaPayment::EFT::CPA005::Debit
  class File
    include CanadaPayment::EFT::CPA005::Debit
    include CanadaPayment::EFT::Helper

    DEFAULT_CURRENCY = "CAD"
    RECORD_LENGTH = 1464

    attr_accessor :segments, :test
    attr_reader   :client_number,
                  :operating_subsidiary,
                  :file_creation_number,
                  :creation_date,
                  :processing_centre,
                  :currency

    def initialize(client_number:,
                   file_creation_number:,
                   processing_centre:,
                   operating_subsidiary: nil,
                   creation_date: Date.Today,
                   currency: DEFAULT_CURRENCY,
                   test: false)
      @segments = []
      @client_number        = client_number
      @operating_subsidiary = operating_subsidiary
      @file_creation_number = file_creation_number
      @creation_date        = creation_date
      @processing_centre    = processing_centre
      @currency             = currency.upcase
      @test                 = test
    end

    # BASE CLASS OVERRIDES

    def file_header
      "$$AA01CPA1464[#{@test ? 'TEST' : 'PROD'}[NL$$"
    end

    def header_record
      'A' +
      zero_padded('1', 9) +
      client_number_with_subsidiary +
      formatted_file_creation_number +
      file_creation_date +
      zero_padded(processing_centre, 5) +
      space_padded(nil, 20) +
      ensure_length(currency, 3).upcase +
      space_padded(nil, 1406)
    end

    def trailer_record
      'Z' +
      zero_padded(trailer_record_number, 9) +
      client_number_with_subsidiary +
      formatted_file_creation_number +
      zero_padded(total_amount_cents, 14) +
      zero_padded(@segments.size, 8) +
      zero_padded(nil, 1418)
    end

    def add_segment(customer_name:, customer_number:, amount_cents:, payment_date:,
                    institution:, transit:, account_number:,
                    transaction_code: nil, client_short_name: nil, client_long_name: nil,
                    client_sundry_info: nil, client_number: nil, operating_subsidiary: nil)

      client_number        ||= @client_number
      operating_subsidiary ||= @operating_subsidiary
      client_short_name    ||= @client_short_name
      client_long_name     ||= @client_long_name

      segment = Segment.new customer_name: customer_name,
                            customer_number: customer_number,
                            amount_cents: amount_cents,
                            payment_date: payment_date,
                            institution: institution,
                            transit: transit,
                            account_number: account_number,
                            transaction_code: transaction_code,
                            client_short_name: client_short_name,
                            client_long_name: client_long_name,
                            client_sundry_info: client_sundry_info,
                            client_number: client_number,
                            operating_subsidiary: operating_subsidiary

      @segments << segment
      segment
    end

    def basic_records
      output = []

      segments.each_with_index do |segment, i|
        output << '' if output[-1].nil? || output[-1].length == RECORD_LENGTH

        # add record header
        if i % 6 == 0
          record_number = (i / 6) + 2
          output[-1] += "D" + zero_padded(record_number, 9) +
                        client_number_with_subsidiary + formatted_file_creation_number
        end

        output[-1] += segment.to_s
      end

      # fill last row with spaces if it doesn't contain 6 transactions
      output[-1] += " " * (RECORD_LENGTH - output[-1].length)

      output
    end

    def to_s
      file_header + "\n" + header_record + "\n" + basic_records.join("\n") + "\n" + trailer_record
    end
    alias_method :output, :to_s

    private

    def total_amount_cents
      @segments.map {|s| s.amount_cents}.reduce(0, :+)
    end

    def basic_record_count
      # Basic records contain 6 segments
      (@segments.count / 6.0).ceil.to_i
    end

    def trailer_record_number
      basic_record_count + 2
    end

    def client_number_with_subsidiary
      ensure_length(client_number, 6) + zero_padded(operating_subsidiary, 4)
    end

    def file_creation_date
      padded_date(creation_date, 6)
    end

    def formatted_file_creation_number
      (test ? 'TEST' : zero_padded(file_creation_number, 4))
    end
  end
end
